@@ -109,11 +109,18 @@ class AbstractElement:
             "ref",
             ("classes", "class"),
             "style",
-            "v_model",
-            "v_if",
-            "v_show",
-            "v_for",
             ("key", ":key"),
+            # default vue.js directives
+            "v_text",
+            "v_html",
+            "v_show",
+            "v_if",
+            "v_else",
+            "v_else_if",
+            "v_for",
+            "v_model",
+            "v_pre",
+            "v_once",
         ]
         self._event_names += [
             "click",
@@ -474,6 +481,7 @@ class Input(AbstractElement):
             "list",
             "autocomplete",
         ]
+        self._event_names += ["change", "input"]
 
 
 class Template(AbstractElement):
@@ -583,3 +591,16 @@ class Triggers(AbstractElement):
         """
         _app = get_app_instance()
         _app.update(ref=self._ref, method="emit", args=[name, *args])
+
+
+class VTKLoading(AbstractElement):
+    """
+    Component to show the 3 spinning partial circles using the ParaView Red/Green/Yellow colors.
+
+    :param message: Message to put below the spinning circles
+    :type message: str
+    """
+
+    def __init__(self, message="", **kwargs):
+        super().__init__("vtk-loading", message=message, **kwargs)
+        self._attr_names += ["message"]
